MODULE crsdiawri
   !!======================================================================
   !!                     ***  MODULE  diawri  ***
   !! Ocean diagnostics :  write ocean output files
   !!=====================================================================
   !! History :  OPA  ! 1991-03  (M.-A. Foujols)  Original code
   !!            4.0  ! 1991-11  (G. Madec)
   !!                 ! 1992-06  (M. Imbard)  correction restart file
   !!                 ! 1992-07  (M. Imbard)  split into diawri and rstwri
   !!                 ! 1993-03  (M. Imbard)  suppress writibm
   !!                 ! 1998-01  (C. Levy)  NETCDF format using ioipsl INTERFACE
   !!                 ! 1999-02  (E. Guilyardi)  name of netCDF files + variables
   !!            8.2  ! 2000-06  (M. Imbard)  Original code (diabort.F)
   !!   NEMO     1.0  ! 2002-06  (A.Bozec, E. Durand)  Original code (diainit.F)
   !!             -   ! 2002-09  (G. Madec)  F90: Free form and module
   !!             -   ! 2002-12  (G. Madec)  merge of diabort and diainit, F90
   !!                 ! 2005-11  (V. Garnier) Surface pressure gradient organization
   !!            3.2  ! 2008-11  (B. Lemaire) creation from old diawri
   !!                 ! 2012-07  (J. Simeon, G. Madec, C. Ethe) Modified for coarsened output
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   crs_dia_wri       : create the standard output files for coarse grid and prep
   !!                       other variables needed to be passed to TOP
   !!----------------------------------------------------------------------
   USE oce             ! ocean dynamics and tracers 
   USE dom_oce         ! ocean space and time domain
   USE zdf_oce         ! ocean vertical physics
   USE ldftra_oce      ! ocean active tracers: lateral physics
   USE ldfdyn_oce      ! ocean dynamics: lateral physics
   USE traldf_iso_grif, ONLY : psix_eiv, psiy_eiv
   USE sol_oce         ! solver variables
   USE sbc_oce         ! Surface boundary condition: ocean fields
   USE sbcrnf
   USE sbc_ice         ! Surface boundary condition: ice fields
   USE sbcssr          ! restoring term toward SST/SSS climatology
   USE phycst          ! physical constants
   USE zdfmxl          ! mixed layer
   USE dianam          ! build name of file (routine)
   USE zdfddm          ! vertical  physics: double diffusion
   USE diahth          ! thermocline diagnostics
   USE lbclnk          ! ocean lateral boundary conditions (or mpp link)
   USE in_out_manager  ! I/O manager
   USE diadimg         ! dimg direct access file format output
   USE diaar5, ONLY :   lk_diaar5
   USE iom
   USE eosbn2
#if defined key_lim2
   USE limwri_2 
#endif
   USE lib_mpp         ! MPP library
   USE timing          ! preformance summary
   USE wrk_nemo        ! working array
   USE crs_dom
   USE crs_iom
   USE crs
   USE crslbclnk
 !  USE crseosbn2
   USE crs_iom

   IMPLICIT NONE
   PRIVATE

   PUBLIC   crs_dia_wri                 ! routines called by step.F90


   !! * Substitutions
#  include "zdfddm_substitute.h90"
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.3 , NEMO Consortium (2010)
   !! $Id $
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE crs_dia_wri( kt )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE crs_dia_wri  ***
      !!                   
      !! ** Purpose :   Basic output of coarsened dynamics and tracer fields 
      !!      NETCDF format is used by default 
      !!      1. Accumulate in time the dimensionally-weighted fields
      !!      2. At time of output, rescale [1] by dimension and time
      !!         to yield the spatial and temporal average.
      !!  See. diawri_dimg.h90, sbcmod.F90
      !!
      !! ** Method  :  use iom_put via crs_iom_put
      !!----------------------------------------------------------------------
      !!
      INTEGER, INTENT( in ) ::   kt      ! ocean time-step index
      !!
      INTEGER                      ::   ji, jj, jk              ! dummy loop indices
      !!
      REAL(wp), POINTER, DIMENSION(:,:)   :: z2d    ! 2D workspace
      REAL(wp), POINTER, DIMENSION(:,:,:) :: z3d    ! 3D workspace
      REAL(wp), POINTER, DIMENSION(:,:,:) :: z3d1   ! 3D workspace
      REAL(wp), POINTER, DIMENSION(:,:,:) :: zfse3t ! 3D workspace for e3
      REAL(wp), POINTER, DIMENSION(:,:,:) :: zfse3u ! 3D workspace for e3
      REAL(wp), POINTER, DIMENSION(:,:,:) :: zfse3v ! 3D workspace for e3
      REAL(wp), POINTER, DIMENSION(:,:,:) :: zfse3w ! 3D workspace for e3
      REAL(wp), POINTER, DIMENSION(:,:)   :: ze1e2u ! 2D workspace
      REAL(wp), POINTER, DIMENSION(:,:)   :: ze1e2v ! 2D workspace
      REAL(wp), POINTER, DIMENSION(:,:,:) :: z3dcrs ! 3D workspace for coarse grid
      REAL(wp), POINTER, DIMENSION(:,:)   :: z2dcrs ! 2D workspace for coarse grid
      INTEGER        :: ialloc, iiki, iikn
      INTEGER, SAVE  :: itsct
      REAL(wp)       :: zdtj, zrtsct
      REAL(wp), ALLOCATABLE, DIMENSION(:,:,:,:), SAVE :: ztsnm

      REAL(wp), ALLOCATABLE, DIMENSION(:,:,:), SAVE :: zum, zvm, zwm  ! 3d work arrays for
      REAL(wp), ALLOCATABLE, DIMENSION(:,:,:), SAVE :: zutm, zusm, zvtm, zvsm, zwtm, zwsm              ! accumulating sums for averaging
      REAL(wp), ALLOCATABLE, DIMENSION(:,:,:), SAVE :: zvolw, zsuru, zsurv, zsurw, zhdivnm
      REAL(wp), ALLOCATABLE, DIMENSION(:,:,:), SAVE :: zrn_absm, zrhopm, zrhdm, zrotbm
      REAL(wp), ALLOCATABLE, DIMENSION(:,:,:), SAVE :: zavtm, zavsm, zuslpm,zvslpm, zwslpim, zwslpjm

      REAL(wp), ALLOCATABLE, DIMENSION(:,:  ), SAVE :: zsshbm, zsshm, zssham, zsshfm                    ! 2d work arrays 
      REAL(wp), ALLOCATABLE, DIMENSION(:,:  ), SAVE :: zsshubm, zsshum, zsshuam, zsurtaux, zutaum       ! for accumulating sums for averaging
      REAL(wp), ALLOCATABLE, DIMENSION(:,:  ), SAVE :: zsshvbm, zsshvm, zsshvam, zsurtauy, zvtaum       
      REAL(wp), ALLOCATABLE, DIMENSION(:,:  ), SAVE :: zhdivbtm, zempm, zemp_bm, zempsm
      REAL(wp), ALLOCATABLE, DIMENSION(:,:  ), SAVE :: zwndmm, zqsrm, zrnfm, zfr_im, zh_rnfm

      !!
      REAL(wp)                                      :: zcoefu, zcoefv, zcoeff, z2dt, z1_2dt, z1_rau0, zraur
      REAL(wp)                                      :: zij, zip1j, zijp1
      REAL(wp), ALLOCATABLE, DIMENSION(:,:)         :: z2dcrsu, z2dcrsv, z2dcrsf
      REAL(wp), ALLOCATABLE, DIMENSION(:,:)         :: zsshub, zsshua, zsshvb, zsshva   ! temp work arrays for instantaneous fields
      REAL(wp), ALLOCATABLE, DIMENSION(:,:,:)       :: zhdiv, zvolt_wgt, zrhd, zrhop, zavt
      REAL(wp), ALLOCATABLE, DIMENSION(:,:)         :: zhdivbt
      REAL(wp), ALLOCATABLE, DIMENSION(:,:)         :: zee_t, zee_f, zee_u, zee_v
      REAL(wp), ALLOCATABLE, DIMENSION(:,:,:)       :: zmut, zmuf
      REAL(wp), ALLOCATABLE, DIMENSION(:,:)         :: zcrs_surtaux_wgt , zcrs_surtauy_wgt
      REAL(wp), ALLOCATABLE, DIMENSION(:,:,:,:)     :: ztsn      
      REAL(wp) ::   zrho_c = 0.01_wp    ! density criterion for mixed layer depth
      REAL(wp) ::   zavt_c = 5.e-4_wp   ! Kz criterion for the turbocline depth

      INTEGER , ALLOCATABLE, DIMENSION(:,:  )       :: imld

      CHARACTER(LEN=80) :: clmode
      !!----------------------------------------------------------------------
      ! 

      IF( nn_timing == 1 )   CALL timing_start('crs_dia_wri')
      ! 
      ! 1. Initialize arrays
      ! --------------------------------------------------------
      CALL wrk_alloc( jpi , jpj , z2d, ze1e2u, ze1e2v )
      CALL wrk_alloc( jpi , jpj , jpk , z3d, z3d1 )
      CALL wrk_alloc( jpi , jpj , jpk , zfse3u, zfse3v )
      CALL wrk_alloc( jpi , jpj , jpk , zfse3t, zfse3w )
      CALL wrk_alloc( jpi_crs , jpj_crs , jpk , z3dcrs )
      CALL wrk_alloc( jpi_crs , jpj_crs , z2dcrs )

      IF ( .NOT. ALLOCATED(ztsnm) ) THEN
         ALLOCATE( ztsnm(jpi_crs,jpj_crs,jpk,jpts), STAT=ialloc )
         IF ( ialloc /= 0 ) CALL ctl_warn('crsdiawri : failed to allocate cumulative 4d array')
      ENDIF

      IF ( .NOT. ALLOCATED(zum) ) THEN
         ALLOCATE( zum(jpi_crs,jpj_crs,jpk) ,   zvm(jpi_crs,jpj_crs,jpk) ,  &
            &    zavtm(jpi_crs,jpj_crs,jpk) , zvolw(jpi_crs,jpj_crs,jpk) ,  &
            &    zsuru(jpi_crs,jpj_crs,jpk) , zsurv(jpi_crs,jpj_crs,jpk) ,  &
            &    zsurw(jpi_crs,jpj_crs,jpk) ,   zwm(jpi_crs,jpj_crs,jpk) ,  &
            &     zutm(jpi_crs,jpj_crs,jpk) ,  zusm(jpi_crs,jpj_crs,jpk) ,  &
            &     zvtm(jpi_crs,jpj_crs,jpk) ,  zvsm(jpi_crs,jpj_crs,jpk) ,  &
            &     zwtm(jpi_crs,jpj_crs,jpk) ,  zwsm(jpi_crs,jpj_crs,jpk) ,  &
            &     zrhopm(jpi_crs,jpj_crs,jpk), zrhdm(jpi_crs,jpj_crs,jpk) ,  &
            &   zhdivnm(jpi_crs,jpj_crs,jpk) ,  STAT=ialloc )
         IF( ialloc /= 0 )   CALL ctl_warn('crsdiawri : failed to allocate cumulative 3d arrays')
      ENDIF

      IF ( .NOT. ALLOCATED(zsshm) ) THEN
         ALLOCATE( zsshm(jpi_crs,jpj_crs) ,   zsshfm(jpi_crs,jpj_crs) , &
            &     zsshbm(jpi_crs,jpj_crs) ,   zssham(jpi_crs,jpj_crs) , &
            &     zsshum(jpi_crs,jpj_crs) ,   zsshvm(jpi_crs,jpj_crs) , &
            &   zhdivbtm(jpi_crs,jpj_crs) ,                             &
            &      zempm(jpi_crs,jpj_crs) ,  zemp_bm(jpi_crs,jpj_crs) , &
            &     zempsm(jpi_crs,jpj_crs) ,                             &
            &     zvtaum(jpi_crs,jpj_crs) ,   zutaum(jpi_crs,jpj_crs) , &
            &   zsurtaux(jpi_crs,jpj_crs) , zsurtauy(jpi_crs,jpj_crs) , &
            &     zwndmm(jpi_crs,jpj_crs) ,    zqsrm(jpi_crs,jpj_crs) , &
            &      zrnfm(jpi_crs,jpj_crs) ,   zfr_im(jpi_crs,jpj_crs) , &
            &    zh_rnfm(jpi_crs,jpj_crs) ,                             &
            &     STAT=ialloc )
         IF( ialloc /= 0 )   CALL ctl_warn('crsdiawri : failed to allocate cumulative 2d arrays')
      ENDIF

      IF ( .NOT. ALLOCATED(z2dcrsu) ) THEN
         ALLOCATE( z2dcrsu(jpi_crs,jpj_crs) , z2dcrsv(jpi_crs,jpj_crs) , &
            &      z2dcrsf(jpi_crs,jpj_crs) , zhdivbt(jpi_crs,jpj_crs) , &
            &      zsshub(jpi_crs,jpj_crs)  , zsshvb(jpi_crs,jpj_crs)  , &
            &      zsshua(jpi_crs,jpj_crs)  , zsshva(jpi_crs,jpj_crs)  , &
            &      zee_t(jpi_crs,jpj_crs)   , zee_u(jpi_crs,jpj_crs)    , &
            &      zee_v(jpi_crs,jpj_crs)   , zee_f(jpi_crs,jpj_crs)    , &
            &      zmut(jpi_crs,jpj_crs,jpk), zmuf(jpi_crs,jpj_crs,jpk)    , &
            &      zcrs_surtaux_wgt(jpi_crs,jpj_crs) , zcrs_surtauy_wgt(jpi_crs,jpj_crs) , &
            &        imld(jpi_crs,jpj_crs)  ,                            &
            &      STAT=ialloc )
         IF( ialloc /= 0 )   CALL ctl_warn('crsdiawri : failed to allocate temp 2d arrays')
      ENDIF
 
      IF ( .NOT. ALLOCATED(zhdiv) ) THEN
         ALLOCATE( zhdiv(jpi_crs,jpj_crs,jpk) , zvolt_wgt(jpi_crs,jpj_crs,jpk) , &
            &       zrhd(jpi_crs,jpj_crs,jpk) ,     zrhop(jpi_crs,jpj_crs,jpk) , &
            &       zavt(jpi_crs,jpj_crs,jpk) , STAT=ialloc )
         IF( ialloc /= 0 )   CALL ctl_warn('crsdiawri : failed to allocate temp 3d arrays')
      ENDIF

      IF ( .NOT. ALLOCATED(ztsn) ) THEN
         ALLOCATE( ztsn(jpi_crs,jpj_crs,jpk,jpts),    STAT=ialloc )
         IF( ialloc /= 0 )   CALL ctl_warn('crsdiawri : failed to allocate temp 4d arrays')
      ENDIF


      ! generic work arrays
      z2d(:,:)      = 0.0
      z3d(:,:,:)    = 0.0
      z3d1(:,:,:)   = 0.0

      ! Depth work arrrays
      zfse3t(:,:,:) = fse3t(:,:,:)
      zfse3u(:,:,:) = fse3u(:,:,:)
      zfse3v(:,:,:) = fse3v(:,:,:)
      zfse3w(:,:,:) = fse3w(:,:,:)

      ! Horizontal divergence work array
      zhdiv(:,:,:) = 0._wp
      zhdivbt(:,:) = 0._wp

      zvolt_wgt(:,:,:) = 1._wp

      ! Sea-surface height work arrays
      zee_t(:,:) = 0._wp    ;   zee_f(:,:) = 0._wp
      zee_u(:,:) = 0._wp    ;   zee_v(:,:) = 0._wp
      zmut(:,:,:) = 0._wp   ;   zmuf(:,:,:) = 0._wp

      ! work arrays
      ze1e2u(:,:) = e1u(:,:) * e2u(:,:)
      ze1e2v(:,:) = e1v(:,:) * e2v(:,:)
 
      IF( kt == nit000  ) THEN
         ! reset work arrays for average computation
         itsct = 0
         !
         zum(:,:,:)   = 0._wp
         zvm(:,:,:)   = 0._wp
         ztsnm(:,:,:,:) = 0._wp
         zwm(:,:,:)   = 0._wp
         zutm(:,:,:)  = 0._wp
         zusm(:,:,:)  = 0._wp
         zvtm(:,:,:)  = 0._wp
         zvsm(:,:,:)  = 0._wp
         zwtm(:,:,:)  = 0._wp
         zwsm(:,:,:)  = 0._wp
         zavtm(:,:,:)  = 0._wp

         zvolw(:,:,:) = 0._wp   ! t-centered grid box volume, masked
         zsuru(:,:,:) = 0._wp   ! east face area, masked
         zsurv(:,:,:) = 0._wp   ! north face area, masked
         zsurw(:,:,:) = 0._wp   ! top face area, masked on gridT
         zsurtaux(:,:) = 0._wp  ! top face area, masked on gridU
         zsurtauy(:,:) = 0._wp  ! top face area, masked on gridV

         zsshm(:,:)   = 0._wp
         zsshum(:,:)  = 0._wp
         zsshvm(:,:)  = 0._wp
         zsshfm(:,:)  = 0._wp
         zsshbm(:,:) = 0._wp
         zsshubm(:,:) = 0._wp
         zsshvbm(:,:) = 0._wp
         zssham(:,:) = 0._wp
         zsshuam(:,:) = 0._wp
         zsshvam(:,:) = 0._wp
         zwndmm(:,:) = 0._wp
         zutaum(:,:) = 0._wp
         zvtaum(:,:) = 0._wp
         zqsrm(:,:) = 0._wp
         zrnfm(:,:) = 0._wp
         zh_rnfm(:,:) = 0._wp
         zfr_im(:,:) = 0._wp

         tsn_crs(:,:,:,:) = 0._wp               ! passive tracer array, now 
         un_crs(:,:,:) = 0._wp                  ! u-velocity
         vn_crs(:,:,:) = 0._wp                  ! v-velocity
         wn_crs(:,:,:) = 0._wp                  ! bottom boundary condition: w(:,:,jpk)=0 (set once for all) 
         sshn_crs(:,:) = 0._wp                  ! sea-surface height now
         ut_crs(:,:,:) = 0._wp                  ! U*T
         us_crs(:,:,:) = 0._wp                  ! U*S
         vt_crs(:,:,:) = 0._wp                  ! V*T
         vs_crs(:,:,:) = 0._wp                  ! V*S
         avt_crs(:,:,:) = 0._wp                  ! avt

         zcrs_surtaux_wgt(:,:) = ze1e2u(:,:) * umask(:,:,1)
         zcrs_surtauy_wgt(:,:) = ze1e2v(:,:) * vmask(:,:,1)

      ENDIF

      ! 2. Coarsen fields and accumulate sums at each time step
      ! --------------------------------------------------------
! jes, TODO. 31 aug 2012
! need to pass the 
! 2D. utau, vtau, wndm, qsr, rnf, fi_i(:,:), h_rnf , nmln, hmld, hmlp, hmlpt
! 3D. rn_abs(:,:,:), rhop, rhd=(rho-rau0)/rau0,  rotb, avt, avs, slopes, 
! DIM?. aht0, ahtu, ahtv, ahtw, ahtt, aeiu, aeiv, aeiw


      itsct = itsct+1     ! counter for the number of time steps 

      ! 2.0  Weights
      zvolw(:,:,:) = zvolw(:,:,:) + crs_volt_wgt(:,:,:)   ! masked ocean volume
      zsurv(:,:,:) = zsurv(:,:,:) + crs_surfv_wgt(:,:,:)  ! masked north face surface area
      zsuru(:,:,:) = zsuru(:,:,:) + crs_surfu_wgt(:,:,:)  ! masked east face surface area
      zsurw(:,:,:) = zsurw(:,:,:) + crs_surfw_wgt(:,:,:)  ! masked top face surface area of gridT
      zsurtaux(:,:) = zsurtaux(:,:) + zcrs_surtaux_wgt(:,:) ! masked top face surface area of gridU
      zsurtauy(:,:) = zsurtauy(:,:) + zcrs_surtauy_wgt(:,:) ! masked top face surface area of gridV

      ! 2.1 Temperature
      !    2.1.1. now-temperature
      z3d(:,:,:) = tsn(:,:,:,jp_tem)
      CALL crsfun( p_e1e2t=e1e2t, cd_type='T', cd_op='VOL', p_cmask=tmask_crs, p_ptmask=tmask, &
         &         p_pfield3d_1=zfse3t, p_pfield3d_2=z3d, p_cfield3d=z3dcrs )
      ztsn(:,:,:,1) = z3dcrs(:,:,:)
      ztsnm(:,:,:,1) = ztsnm(:,:,:,1) + ztsn(:,:,:,1) 

      ! 2.2 Salinity
      !    2.2.1. now-salinity
      z3d(:,:,:) = tsn(:,:,:,jp_sal)   ; z3dcrs(:,:,:) = 0.0 
      CALL crsfun( p_e1e2t=e1e2t, cd_type='T', cd_op='VOL', p_cmask=tmask_crs, p_ptmask=tmask, &
         &         p_pfield3d_1=zfse3t, p_pfield3d_2=z3d, p_cfield3d=z3dcrs )
      ztsn(:,:,:,2) = z3dcrs(:,:,:)
      ztsnm(:,:,:,2) = ztsnm(:,:,:,2) + ztsn(:,:,:,2) 

      !    2.5.9 Vertical eddy diffusion coefficient (avt) on gridW
      z3d(:,:,:) = avt(:,:,:)   ;  z3dcrs(:,:,:) = 0.0 
      CALL crsfun( p_e1e2t=e1e2t, cd_type='T', cd_op='SUM', p_cmask=tmask_crs, p_ptmask=tmask, &
         &         p_pfield3d_1=zfse3t, p_pfield3d_2=z3d, p_cfield3d=z3dcrs )
      zavtm(:,:,:) = zavtm(:,:,:) + z3dcrs(:,:,:)       

      ! 2.3. U-Velocity
      !    2.3.1. U
      z3dcrs(:,:,:) = 0.0
      CALL crsfun( p_e1_e2=e2u, cd_type='U', psgn=-1.0, p_pmask=umask, &
         &         p_fse3=zfse3u, p_pfield=un, p_surf_crs=crs_surfu_wgt, p_cfield3d=z3dcrs )
      zum(:,:,:) = z3dcrs(:,:,:)
      !    2.3.2.  UT, US
      z3d(:,:,:) = 0.e0 ; z3d1(:,:,:) = 0.0
      DO jk = 1, jpkm1
         DO jj = 2, jpjm1
            DO ji = 2, jpim1   
               z3d(ji,jj,jk)  = un(ji,jj,jk) * 0.5 * ( tsn(ji,jj,jk,jp_tem) + tsn(ji+1,jj,jk,jp_tem) ) 
               z3d1(ji,jj,jk) = un(ji,jj,jk) * 0.5 * ( tsn(ji,jj,jk,jp_sal) + tsn(ji+1,jj,jk,jp_sal) ) 
            END DO
         END DO
      END DO
      !   UT
      z3dcrs(:,:,:) = 0.0
      CALL crsfun(  p_e1_e2=e2u, cd_type='U', psgn=-1.0, p_pmask=umask, &
         &         p_fse3=zfse3u, p_pfield=z3d, p_cfield3d=z3dcrs )
      zutm(:,:,:) = zutm(:,:,:) + z3dcrs(:,:,:)
      !   US
      z3dcrs(:,:,:) = 0.0
      CALL crsfun(  p_e1_e2=e2u, cd_type='U', psgn=-1.0, p_pmask=umask, &
         &         p_fse3=zfse3u, p_pfield=z3d1, p_cfield3d=z3dcrs )
      zusm(:,:,:) = zusm(:,:,:) + z3dcrs(:,:,:)


      ! 2.4. V-Velocity
      !    2.4.1. V
      z3dcrs(:,:,:) = 0.0
      CALL crsfun( p_e1_e2=e1v, cd_type='V', psgn=-1.0, p_pmask=vmask, &
         &         p_fse3=zfse3v, p_pfield=vn, p_surf_crs=crs_surfv_wgt, p_cfield3d=z3dcrs )
      zvm(:,:,:) = z3dcrs(:,:,:)
      !    2.4.2  VT, VS
      z3d(:,:,:) = 0.e0 ; z3d1(:,:,:) = 0.0
      DO jk = 1, jpkm1
         DO jj = 2, jpjm1
            DO ji = 2, jpim1   
               z3d(ji,jj,jk)  = vn(ji,jj,jk) * 0.5 * ( tsn(ji,jj,jk,jp_tem) + tsn(ji,jj+1,jk,jp_tem) ) 
               z3d1(ji,jj,jk) = vn(ji,jj,jk) * 0.5 * ( tsn(ji,jj,jk,jp_sal) + tsn(ji,jj+1,jk,jp_sal) ) 
            END DO
         END DO
      END DO
      !   VT
      z3dcrs(:,:,:) = 0.0
      CALL crsfun( p_e1_e2=e1v, cd_type='V', psgn=-1.0, p_pmask=vmask, &
         &         p_fse3=zfse3v, p_pfield=z3d, p_cfield3d=z3dcrs )
      zvtm(:,:,:) = zvtm(:,:,:) + z3dcrs(:,:,:)
      !   VS
      z3dcrs(:,:,:) = 0.0
      CALL crsfun( p_e1_e2=e1v, cd_type='V', psgn=-1.0, p_pmask=vmask, &
         &         p_fse3=zfse3v, p_pfield=z3d1, p_cfield3d=z3dcrs )
      zvsm(:,:,:) = zvsm(:,:,:) + z3dcrs(:,:,:)

      ! 2.5. Surface boundary conditions
      !    2.5.1 Evaporation minus Precipitation (emp)
      z2d(:,:) = emp_b(:,:)   ; z2dcrs(:,:) = 0._wp
      CALL crsfun( p_e1e2t=e1e2t, cd_type='T', cd_op='ARE', p_cmask=tmask_crs, p_ptmask=tmask, &
         &         p_pfield2d=z2d, p_cfield2d=z2dcrs )   
      zemp_bm(:,:) = zemp_bm(:,:) + z2dcrs     ! summation of emp_b_crs
      !
      z2d(:,:) = emp(:,:)   ; z2dcrs(:,:) = 0._wp
      CALL crsfun( p_e1e2t=e1e2t, cd_type='T', cd_op='ARE', p_cmask=tmask_crs, p_ptmask=tmask, &
         &         p_pfield2d=z2d, p_cfield2d=z2dcrs )
      zempm(:,:) = zempm(:,:) + z2dcrs     ! summation of emp_crs
      !
      z2d(:,:) = emps(:,:)   ; z2dcrs(:,:) = 0._wp
      CALL crsfun( p_e1e2t=e1e2t, cd_type='T', cd_op='ARE', p_cmask=tmask_crs, p_ptmask=tmask, &
         &         p_pfield2d=z2d, p_cfield2d=z2dcrs )
      zempsm(:,:) = zempsm(:,:) + z2dcrs     ! summation of emps_crs
      !         terms needed for ssh calculation
      z2d(:,:) = emp_b(:,:)-emp(:,:)
      CALL crsfun( p_e1e2t=e1e2t, cd_type='T', cd_op='ARE', p_cmask=tmask_crs, p_ptmask=tmask, &
         &         p_pfield2d=z2d, p_cfield2d=del_emp_crs )
      z2d(:,:) = emp_b(:,:)+emp(:,:)
      CALL crsfun( p_e1e2t=e1e2t, cd_type='T', cd_op='ARE', p_cmask=tmask_crs, p_ptmask=tmask, &
         &         p_pfield2d=z2d, p_cfield2d=sum_emp_crs )
      !
      !    2.5.2 Wind Stress (utau, vtau) 
      !          Taux
      z2d(:,:) = utau(:,:)   ; z2dcrs(:,:) = 0._wp
      CALL crsfun( p_e1e2t=ze1e2u, cd_type='U', cd_op='SUM', p_cmask=umask_crs, p_ptmask=umask, &
         &         p_pfield2d=z2d, p_cfield2d=z2dcrs )
      zutaum(:,:) = zutaum(:,:) + z2dcrs(:,:)      
      !          Tauy
      z2d(:,:) = vtau(:,:)   ; z2dcrs(:,:) = 0._wp
      CALL crsfun( p_e1e2t=ze1e2v, cd_type='V', cd_op='SUM', p_cmask=vmask_crs, p_ptmask=vmask, &
         &         p_pfield2d=z2d, p_cfield2d=z2dcrs )
      zvtaum(:,:) = zvtaum(:,:) + z2dcrs(:,:)
 
      !    2.5.3 Scalar Wind (wndm)
      z2d(:,:) = wndm(:,:)   ; z2dcrs(:,:) = 0._wp
      CALL crsfun( p_e1e2t=e1e2t, cd_type='T', cd_op='SUM', p_cmask=tmask_crs, p_ptmask=tmask, &
         &         p_pfield2d=z2d, p_cfield2d=z2dcrs )
      zwndmm(:,:) = zwndmm(:,:) + z2dcrs(:,:)

      !    2.5.4 Surface shortwave radiative flux (qsr)
      z2d(:,:) = qsr(:,:)   ; z2dcrs(:,:) = 0._wp
      CALL crsfun( p_e1e2t=e1e2t, cd_type='T', cd_op='SUM', p_cmask=tmask_crs, p_ptmask=tmask, &
         &         p_pfield2d=z2d, p_cfield2d=z2dcrs )
      zqsrm(:,:) = zqsrm(:,:) + z2dcrs(:,:)

      !    2.5.5 Runoff freshwater flux
      z2d(:,:) = rnf(:,:)   ; z2dcrs(:,:) = 0._wp
      CALL crsfun( p_e1e2t=e1e2t, cd_type='T', cd_op='SUM', p_cmask=tmask_crs, p_ptmask=tmask, &
         &         p_pfield2d=z2d, p_cfield2d=z2dcrs )
      zrnfm(:,:) = zrnfm(:,:) + z2dcrs(:,:)

      !    2.5.6 Depth of runoff in meters h_rnf(:,:))
      z2d(:,:) = h_rnf(:,:) ; z2dcrs(:,:) = 0._wp
      CALL crsfun(p_e1e2t=e1e2t, cd_type='T', cd_op='SUM', p_cmask=tmask_crs, p_ptmask=tmask, &
         &         p_pfield2d=z2d, p_cfield2d=z2dcrs )
      zh_rnfm(:,:) = zh_rnfm(:,:) + z2dcrs(:,:)

      !    2.5.7 Sea-ice fraction (fr_i)
      z2d(:,:) = fr_i(:,:)   ; z2dcrs(:,:) = 0._wp
      CALL crsfun( p_e1e2t=e1e2t, cd_type='T', cd_op='SUM', p_cmask=tmask_crs, p_ptmask=tmask, &
         &         p_pfield2d=z2d, p_cfield2d=z2dcrs )
      zfr_im(:,:) = zfr_im(:,:) + z2dcrs(:,:)



      !    2.5.8 Potential density (rhop) 
! jes need to declare rho cumulative arrays
  !    CALL crs_eos( ztsn, zrhd, zrhop )

      !! Save the parent grid information
      jpi_full    = jpi
      jpj_full    = jpj
      jpim1_full  = jpim1
      jpjm1_full  = jpjm1

      !! Switch to coarse grid domain
      jpi    = jpi_crs
      jpj    = jpj_crs
      jpim1  = jpi_crsm1
      jpjm1  = jpj_crsm1

      CALL eos( ztsn, zrhd, zrhop )

      !! Return to parent grid domain
      jpi    = jpi_full 
      jpj    = jpj_full
      jpim1  = jpim1_full
      jpjm1  = jpjm1_full

      zrhdm(:,:,:) = zrhdm(:,:,:) + zrhd
      zrhopm(:,:,:) = zrhopm(:,:,:) + zrhop

      ! 2.6. Diagnostic variables: horizontal divergence and vertical velocity
      !    2.6.1 ocean depth at u- v- points
      ! hu_crs(:,:) = ssh_un_crs(:,:)                ! now ocean depth (at u- and v-points)
      ! hv_crs(:,:) = ssh_vn_crs(:,:)
      !                                            ! now masked inverse of the ocean depth (at u- and v-points)
      ! jes. not hur_crs, huv_crs are needed. remove?
      !hur_crs(:,:) = umask_crs(:,:,1) / ( hu_crs(:,:) + 1._wp - umask_crs(:,:,1) )
      !hvr_crs(:,:) = vmask_crs(:,:,1) / ( hv_crs(:,:) + 1._wp - vmask_crs(:,:,1) )
      !
      !    2.6.2. Horizontal divergence
      z2dt = 2._wp * rdt                              ! set time step size (Euler/Leapfrog)
      IF( neuler == 0 .AND. kt == nit000 )   z2dt = rdt
      z1_2dt = 0.e0 / (z2dt)
      !
      WHERE ( crs_volt_wgt /= 0 ) zvolt_wgt(:,:,:) = 1.0/crs_volt_wgt(:,:,:)
      !
      fse3t_crs(:,:,:) = e3t_crs(:,:,:)
      IF ( lk_vvl ) fse3t_crs(:,:,:) = fse3t_n_crs(:,:,:)

      zhdivbt(:,:) = 0.0
      DO jk = 1, jpkm1
         z2dcrsu(:,:) = 0.0; z2dcrsv(:,:) = 0.0   
         DO ji = 2, jpi_crsm1
            DO jj = 2, jpj_crsm1

               ! Horizontal divergence ( following OPA_SRC/DYN/divcur.F90 ) 
               ! with partial steps and/or variable layer thicknesses for W
               z2dcrsu(ji,jj) =  ( zum(ji,  jj,jk) * crs_surfu_wgt(ji,  jj,jk)  ) &
                  &            - ( zum(ji-1,jj,jk) * crs_surfu_wgt(ji-1,jj,jk)  )
               z2dcrsv(ji,jj) =  ( zvm(ji,jj,  jk) * crs_surfv_wgt(ji,jj  ,jk)  ) &
                  &            - ( zvm(ji,jj-1,jk) * crs_surfv_wgt(ji,jj-1,jk)  )

               zhdiv(ji,jj,jk) = ( z2dcrsu(ji,jj) + z2dcrsv(ji,jj) ) * zvolt_wgt(ji,jj,jk) 
 
            ENDDO
         ENDDO

         ! Horizontal barotropic divergence for ssh ( following OPA_SRC/DYN/sshwzv.F90 )

         zhdivbt(:,:) =  zhdivbt(:,:) + fse3t_crs(:,:,jk) * zhdiv(:,:,jk) 

      ENDDO

      CALL crs_lbc_lnk( 'T', 1.0, pt3d1=zhdiv )
      CALL crs_lbc_lnk( 'T', 1.0, pt3d1=zhdivbt )
      ! accumulate 
      zhdivnm(:,:,:) = zhdivnm(:,:,:) + zhdiv
      zhdivbtm(:,:) = zhdivbtm(:,:) + zhdivbt


      !
      !    2.6.3. Sea-surface Height  ( See DOM/istate.F90: ssh init; OPA_SRC/DYN/sshwzv.F90: ssh update )
      !       set some temp variables
      z2dcrs(:,:) = 0.0; z2dcrsf(:,:) = 0.0
      zcoefu = 0.0; zcoefv = 0.0; zcoeff = 0.0; 
      zraur = 1._wp / rau0
      z1_rau0 = 0.5 / rau0
      !       sshn, sshb to accumulate
      DO jj = 2, jpj_crsm1                                      ! leap-frog on ssh_e
         DO ji = 2, jpi_crsm1
            z2dcrs(ji,jj) = ( ssh_b_crs(ji,jj) - z2dt * ( zraur * ( del_emp_crs(ji,jj) ) + zhdivbt(ji,jj) ) ) * tmask_crs(ji,jj,1) 
         END DO
      END DO
      CALL crs_lbc_lnk('T',1.0,pt2d=z2dcrs)   
      zsshm(:,:)  = zsshm(:,:)  + z2dcrs(:,:)
      zsshbm(:,:) = zsshbm(:,:) + ssh_b_crs(:,:)

      !
      !       sshu, sshv, sshf, for vvl case. Calculate then accumulate the fields for time-mean
      !
            DO jj = 1, jpj_crsm1
               DO ji = 1, jpi_crsm1                    
                  zcoefu = 0.5  * umask_crs(ji,jj,1) / ( e1u_crs(ji,jj) * e2u_crs(ji,jj) )
                  zcoefv = 0.5  * vmask_crs(ji,jj,1) / ( e1v_crs(ji,jj) * e2v_crs(ji,jj) )
                  zij   =  e1e2t_crs(ji  ,jj  ) * z2dcrs(ji  ,jj  ) 
                  zip1j =  e1e2t_crs(ji+1,jj  ) * z2dcrs(ji+1,jj  ) 
                  zijp1 =  e1e2t_crs(ji  ,jj+1) * z2dcrs(ji  ,jj+1) 
              
                  ssh_un_crs(ji,jj) = zcoefu * ( zij + zip1j )
                  ssh_vn_crs(ji,jj) = zcoefv * ( zij + zijp1 )
         IF( lk_vvl ) THEN
                  zsshub(ji,jj) = zcoefu * ( e1e2t_crs(ji,  jj) * ssh_b_crs(ji  ,jj)   &
                     &                        +  e1e2t_crs(ji+1,jj) * ssh_b_crs(ji+1,jj) )
                  zsshvb(ji,jj) = zcoefv * ( e1e2t_crs(ji,  jj) * ssh_b_crs(ji,  jj)   &
                     &                        +  e1e2t_crs(ji,jj+1) * ssh_b_crs(ji,jj+1) )
         ENDIF
               END DO
            END DO

            DO jj = 1, jpj_crsm1
               DO ji = 1, jpi_crsm1     
                  z2dcrsf(ji,jj) = 0.5  * umask_crs(ji,jj,1) * umask_crs(ji,jj+1,1)                   &
                       &               / ( e1f_crs(ji,jj  ) * e2f_crs(ji,jj  ) )                     &
                       &               * ( e1u_crs(ji,jj  ) * e2u_crs(ji,jj  ) * ssh_un_crs(ji,jj  )     &
                       &                 + e1u_crs(ji,jj+1) * e2u_crs(ji,jj+1) * ssh_un_crs(ji,jj+1) )
               END DO
            END DO
      !     accumulate fields in time
            CALL crs_lbc_lnk( 'U', 1., pt2d=ssh_un_crs )   ! sshun_crs
            CALL crs_lbc_lnk( 'V', 1., pt2d=ssh_vn_crs )   ! sshvn_crs
            CALL crs_lbc_lnk('F', 1., pt2d=z2dcrsf )
            zsshum(:,:)  = zsshum(:,:)  + ssh_un_crs(:,:)
            zsshvm(:,:)  = zsshvm(:,:)  + ssh_vn_crs(:,:)
            zsshfm(:,:)  = zsshfm(:,:)  + z2dcrsf(:,:)
         IF( lk_vvl ) THEN
            CALL crs_lbc_lnk( 'U', 1., pt2d=zsshub )
            CALL crs_lbc_lnk( 'V', 1., pt2d=zsshvb )
            zsshubm(:,:) = zsshubm(:,:) + zsshub(:,:) 
            zsshvbm(:,:) = zsshvbm(:,:) + zsshvb(:,:) 
         ENDIF
      !       update sshb to sshn
      ssh_b_crs(:,:) = z2dcrs(:,:) 
      !       update ssha from sshn
      ssh_a_crs(:,:) = (  z2dcrs(:,:) - z2dt * ( z1_rau0 * ( sum_emp_crs(:,:) ) + zhdivbt(:,:) )  ) * tmask(:,:,1)
      CALL crs_lbc_lnk('T', 1.0, pt2d=ssh_a_crs)
      zssham(:,:) = zssham(:,:) + ssh_a_crs(:,:)
      !       accumulate fields in time and update sshua from sshun
      IF ( lk_vvl ) THEN
         DO jj = 1, jpj_crsm1
            DO ji = 1, jpi_crsm1     
               zsshua(ji,jj) = 0.5  * umask_crs(ji,jj,1) / ( e1u_crs(ji  ,jj) * e2u_crs(ji  ,jj) )                   &
                  &                                  * ( e1t_crs(ji  ,jj) * e2t_crs(ji  ,jj) * ssh_a_crs(ji  ,jj)     &
                  &                                    + e1t_crs(ji+1,jj) * e2t_crs(ji+1,jj) * ssh_a_crs(ji+1,jj) )
               zsshva(ji,jj) = 0.5  * vmask_crs(ji,jj,1) / ( e1v_crs(ji,jj  ) * e2v_crs(ji,jj  ) )                   &
                  &                                  * ( e1t_crs(ji,jj  ) * e2t_crs(ji,jj  ) * ssh_a_crs(ji,jj  )     &
                  &                                    + e1t_crs(ji,jj+1) * e2t_crs(ji,jj+1) * ssh_a_crs(ji,jj+1) )
            END DO
         END DO

         CALL crs_lbc_lnk( 'U', 1., pt2d=zsshua )
         CALL crs_lbc_lnk( 'V', 1., pt2d=zsshva )

         zsshuam(:,:) = zsshuam(:,:) + zsshua(:,:) 
         zsshvam(:,:) = zsshvam(:,:) + zsshva(:,:)     
      ENDIF
      !
      !    2.5.2.  W-velocity ( See OPA_SRC/DYN/sshwzv.F90 )
      IF ( lk_vvl ) THEN
         !                                 !==  mu computation  ==!
         zee_t(:,:) = e3t_crs(:,:,1)       ! Lower bound : thickness of the first model level
         zee_f(:,:) = e3f_crs(:,:,1)      
         zee_u(:,:) = e3u_crs(:,:,1)      
         zee_v(:,:) = e3v_crs(:,:,1)      
         !
         DO jk = 2, jpkm1                  ! Sum of the masked vertical scale factors
            zee_t(:,:) = zee_t(:,:) + e3t_crs(:,:,jk) * tmask_crs(:,:,jk)
            zee_u(:,:) = zee_u(:,:) + e3u_crs(:,:,jk) * umask_crs(:,:,jk)
            zee_v(:,:) = zee_v(:,:) + e3v_crs(:,:,jk) * vmask_crs(:,:,jk)
            DO jj = 1, jpj_crsm1                      ! f-point : fmask=shlat at coasts, use the product of umask
               zee_f(:,jj) = zee_f(:,jj) + e3f_crs(:,jj,jk) *  umask_crs(:,jj,jk) * umask_crs(:,jj+1,jk)
            END DO
         END DO
         !                                 ! Compute and mask the inverse of the local depth at T, U, V and F points
         zee_t(:,:) = 1._wp / zee_t(:,:) * tmask_crs(:,:,1)
         zee_u(:,:) = 1._wp / zee_u(:,:) * umask_crs(:,:,1)
         zee_v(:,:) = 1._wp / zee_v(:,:) * vmask_crs(:,:,1)
         DO jj = 1, jpj_crsm1                               ! f-point case fmask cannot be used
            zee_f(:,jj) = 1._wp / zee_f(:,jj) * umask_crs(:,jj,1) * umask_crs(:,jj+1,1)
         END DO
         CALL crs_lbc_lnk( 'F', 1., pt2d=zee_f )                 ! lateral boundary condition on ee_f
         !
         DO jk = 1, jpk                    ! mu coefficients
            zmut(:,:,jk) = zee_t(:,:) * tmask_crs(:,:,jk)     ! T-point at T levels
         END DO
         DO jk = 1, jpk                                 ! F-point : fmask=shlat at coasts, use the product of umask
            DO jj = 1, jpj_crsm1
               zmuf(:,jj,jk) = zee_f(:,jj) * umask_crs(:,jj,jk) * umask_crs(:,jj+1,jk)   ! at T levels
            END DO
            zmuf(:,jpj,jk) = 0._wp
         END DO
         CALL crs_lbc_lnk( 'F', 1., pt3d1=zmuf )                   ! lateral boundary condition

         DO jk = 1, jpk                    ! mu coefficients
            fse3t_crs(:,:,jk)   = e3t_crs(:,:,jk)
            fse3t_n_crs(:,:,jk) = e3t_crs(:,:,jk) + z2dcrs(:,:)*zmut(:,:,jk)
            fse3t_b_crs(:,:,jk) = fse3t_b_crs(:,:,jk)
            fse3t_a_crs(:,:,jk) = e3t_crs(:,:,jk) + ssh_a_crs(:,:)*zmut(:,:,jk)
         ENDDO
! jes. 30 aug 2012.  Need to add here fse3u, fse3v, fse3f - n,b,a
      ELSE
         fse3t_crs = e3t_crs
!         fse3w_crs = e3w_crs
!         fse3u_crs = e3u_crs
!         fse3v_crs = e3v_crs
      ENDIF
      !
      z3dcrs(:,:,:) = 0.0      
      IF ( lk_vvl ) THEN
         DO jk = jpkm1, 1, -1                             ! integrate from the bottom the hor. divergence
            z3dcrs(:,:,jk) = z3dcrs(:,:,jk+1) -   fse3t_n_crs(:,:,jk) * zhdiv(:,:,jk)        &
               &                      - ( fse3t_a_crs(:,:,jk) - fse3t_b_crs(:,:,jk) )    &
               &                         * tmask_crs(:,:,jk) * z1_2dt
         END DO
      ELSE
         DO jk = jpkm1, 1, -1                             ! calculate from the bottom-up
            z3dcrs(:,:,jk) = z3dcrs(:,:,jk+1) - fse3t_crs(:,:,jk) * zhdiv(:,:,jk)        
         ENDDO
      ENDIF
      CALL crs_lbc_lnk( 'T', 1.0, pt3d1=z3dcrs )
      zwm(:,:,:) = zwm(:,:,:) + z3dcrs(:,:,:)    

      ! 2.5.3 Slopes  for slopes look at OPA_SRC/DYN/dynldf_iso.F90 and OPA_SRC/LDF/ldfslp.F90


      ! 3. Output of some dynamics and tracer fields and selected fields
      !   * Within the nn_fcrs frequence for iom 
      ! --------------------------------------------------------
      !
      CALL iom_setkt( kt + nn_fcrs - 1 )    ! in crs, iom_put is called every nn_fcrs time step(s)
 
      IF( MOD( kt-1, nn_fcrs ) == 0 ) THEN

         ! It is time to dump some fields
         ! compute average by scaling the sum by spatial dimension then time
         ! OR by time only

         ! 3.1  Ocean fields
         !    3.1.1 Weight for time average
         zdtj=rdt/86400.   ! time step in days
         WRITE(clmode,'(f5.1,a)' ) zdtj,' days average'
         WRITE(numout,*) 'crsdiawri.', clmode
         zrtsct = 1.0/REAL(itsct, wp)
         !
         !    3.1.2 Weights for spatial averages
         WHERE ( zvolw /= 0 ) zvolw(:,:,:) = 1.0/zvolw(:,:,:)
         WHERE ( zsurv /= 0 ) zsurv(:,:,:) = 1.0/zsurv(:,:,:)
         WHERE ( zsuru /= 0 ) zsuru(:,:,:) = 1.0/zsuru(:,:,:)
         WHERE ( zsurw /= 0 ) zsurw(:,:,:) = 1.0/zsurw(:,:,:)
         WHERE ( zsurtaux /= 0 ) zsurtaux(:,:) = 1.0 / zsurtaux(:,:)
         WHERE ( zsurtauy /= 0 ) zsurtauy(:,:) = 1.0 / zsurtauy(:,:)

         !    3.1.3 Temperature
         tsn_crs(:,:,:,1) = ztsnm(:,:,:,1) * zrtsct ! volume-weighted- , time- mean  
         z2dcrs(:,:) = tsn_crs(:,:,1,1)
         CALL crs_iom_put( "toce_crs"   , pv_r3d=tsn_crs(:,:,:,1)  )    ! temperature
         CALL crs_iom_put( "sst_crs"    , pv_r2d=z2dcrs            )    ! sst
         !
         !    3.1.4 Salinity
         tsn_crs(:,:,:,2) = ztsnm(:,:,:,2) * zrtsct ! volume-weighted- , time- mean 
         z2dcrs(:,:) = tsn_crs(:,:,1,2)
         CALL crs_iom_put( "soce_crs"   , pv_r3d=tsn_crs(:,:,:,2)   )   !  salinity
         CALL crs_iom_put( "sss_crs"    , pv_r2d=z2dcrs             )   !  sss
         !

         !    3.1.5 U-velocity
         un_crs(:,:,:) = zum(:,:,:) * zrtsct ! area-weighted- , time- mean 
         CALL crs_iom_put( "uoce_crs"   , pv_r3d=un_crs             )   ! i-current 
         ut_crs(:,:,:) = zutm(:,:,:) * zsuru(:,:,:) ! area-weighted- , time- mean      
         CALL crs_iom_put( "uocet_crs"     , pv_r3d=ut_crs             )   ! uT
         us_crs(:,:,:) = zusm(:,:,:) * zsuru(:,:,:) ! area-weighted- , time- mean      
         CALL crs_iom_put( "uoces_crs"     , pv_r3d=us_crs             )   ! uS

         !    3.1.6 V-velocity
         vn_crs(:,:,:) = zvm(:,:,:) * zrtsct ! area-weighted- , time- mean 
         CALL crs_iom_put( "voce_crs"   , pv_r3d=vn_crs                   )    ! j-current
         vt_crs(:,:,:) = zvtm(:,:,:) * zsurv(:,:,:) ! area-weighted- , time- mean      
         CALL crs_iom_put( "vocet_crs"     , pv_r3d=vt_crs             )   ! vT
         vs_crs(:,:,:) = zvsm(:,:,:) * zsurv(:,:,:) ! area-weighted- , time- mean      
         CALL crs_iom_put( "voces_crs"     , pv_r3d=vs_crs             )   ! vS

         !    3.1.7 W-velocity
         wn_crs(:,:,:) = zwm(:,:,:) * zrtsct ! area-weighted- , time- mean 
         CALL crs_iom_put( "woce_crs"   , pv_r3d=wn_crs                   )    ! W-velocity

         !    3.1.8 Horizontal divergence
         hdivn_crs(:,:,:) = zhdivnm(:,:,:) * zrtsct
         CALL crs_iom_put( "hdivn_crs"   , pv_r3d=hdivn_crs            )    ! hdiv
         hdivbt_crs(:,:) = zhdivbtm(:,:) * zrtsct

         !    3.1.9 Sea-surface Height
         sshn_crs(:,:) = zsshm(:,:)  * zrtsct
         sshb_crs(:,:) = zsshbm(:,:) * zrtsct
         ssha_crs(:,:) = zssham(:,:) * zrtsct

         IF( lk_vvl ) THEN                    ! now Sea SSH at u-, v-, f-points (vvl case only)
            sshun_crs(:,:) = zsshum(:,:)  * zrtsct                              ! sshun 
            sshub_crs(:,:) = zsshubm(:,:) * zrtsct                              ! sshub 
            sshua_crs(:,:) = zsshuam(:,:) * zrtsct                              ! sshua 
            sshvn_crs(:,:) = zsshvm(:,:)  * zrtsct                              ! sshvn 
            sshvb_crs(:,:) = zsshvbm(:,:) * zrtsct                              ! sshvb 
            sshva_crs(:,:) = zsshvam(:,:) * zrtsct                              ! sshva 
            sshfn_crs(:,:) = zsshfm(:,:) * zrtsct                               ! sshfn 
         ENDIF
         CALL crs_iom_put( "ssh_crs"   , pv_r2d=sshn_crs                  )   ! ssh output 


         !    3.1.10 Potential density
         rhop_crs(:,:,:) = zrhopm(:,:,:) * zrtsct
         rhd_crs(:,:,:)  =  zrhdm(:,:,:) * zrtsct 

         !    3.1.10 Vertical eddy diffusivity
         avt_crs(:,:,:) = zavtm(:,:,:) * zsurw(:,:,:) 

         !    3.1.11  Mixed-layer diagnostics
         !            Recalculated following OPA_SRC/ZDF/zdfmxl.F90
         !   
         ! w-level of the mixing and mixed layers
         nmln_crs(:,:) = mbkt_crs(:,:) + 1        ! Initialization to the number of w ocean point
         imld(:,:) = mbkt_crs(:,:) + 1
         DO jk = jpkm1, nlb10, -1         ! from the bottom to nlb10
            DO jj = 1, jpj_crs
               DO ji = 1, jpi_crs
                  IF( rhop_crs(ji,jj,jk) > rhop_crs(ji,jj,nla10) + zrho_c )   nmln_crs(ji,jj) = jk      ! Mixed layer
                  IF(  avt_crs(ji,jj,jk) < zavt_c                     )           imld(ji,jj) = jk      ! Turbocline
               END DO
            END DO
         END DO
         ! depth of the mixing and mixed layers
         DO jj = 1, jpj_crs
            DO ji = 1, jpi_crs
               iiki = imld(ji,jj)
               iikn = nmln_crs(ji,jj)
               ! jes. TODO.  update here gdep for vvl case
               hmld_crs (ji,jj) = gdepw_crs(ji,jj,iiki  ) * tmask_crs(ji,jj,1)    ! Turbocline depth
               hmlp_crs (ji,jj) = gdepw_crs(ji,jj,iikn  ) * tmask_crs(ji,jj,1)    ! Mixed layer depth
               hmlpt_crs(ji,jj) = gdept_crs(ji,jj,iikn-1)                         ! depth of the last T-point inside the mixed layer
            END DO
         END DO


         ! 3.2. Surface fluxes
         !    3.2.1. Evaporation minus Precipitation
         emp_crs(:,:)   = zempm(:,:)   * zrtsct                     
         emp_b_crs(:,:) = zemp_bm(:,:) * zrtsct                     
         emps_crs(:,:)  = zempsm(:,:)  * zrtsct                     
         !    3.2.2. Wind stresses, Scalar Wind
         utau_crs(:,:) = zutaum(:,:) * zsurtaux(:,:)
         vtau_crs(:,:) = zvtaum(:,:) * zsurtauy(:,:)
         wndm_crs(:,:) = zwndmm(:,:) * zsurw(:,:,1)
         !    3.2.3. Shortwave radiative flux
         qsr_crs(:,:) = zqsrm(:,:) * zsurw(:,:,1)
         !    3.2.4. Runoff freshwater flux
         rnf_crs(:,:) = zrnfm(:,:) * zsurw(:,:,1)
         !    3.2.4. Depth of freshwater flux input
         h_rnf_crs(:,:) = zh_rnfm(:,:) * zrtsct
         !    3.2.5. Ice fraction
         fr_i_crs(:,:) = zfr_im(:,:) * zsurw(:,:,1)


         ! Reset work arrays and counter to 0 after writing
         !
         itsct = 0
         !
         zum(:,:,:)   = 0._wp
         zvm(:,:,:)   = 0._wp
         ztsnm(:,:,:,:) = 0._wp
         zwm(:,:,:)   = 0._wp
         zutm(:,:,:)  = 0._wp
         zusm(:,:,:)  = 0._wp
         zvtm(:,:,:)  = 0._wp
         zvsm(:,:,:)  = 0._wp
         zwtm(:,:,:)  = 0._wp
         zwsm(:,:,:)  = 0._wp

         zvolw(:,:,:) = 0._wp
         zsuru(:,:,:) = 0._wp
         zsurv(:,:,:) = 0._wp
         zsurw(:,:,:) = 0._wp   
         zsurtaux(:,:) = 0._wp
         zsurtauy(:,:) = 0._wp

         zsshm(:,:) = 0._wp
         zsshum(:,:) = 0._wp
         zsshvm(:,:) = 0._wp
         zsshfm(:,:) = 0._wp
         zsshbm(:,:) = 0._wp
         zsshubm(:,:) = 0._wp
         zsshvbm(:,:) = 0._wp
         zssham(:,:) = 0._wp
         zsshuam(:,:) = 0._wp
         zsshvam(:,:) = 0._wp
         zwndmm(:,:) = 0._wp
         zutaum(:,:) = 0._wp
         zvtaum(:,:) = 0._wp
         zqsrm(:,:) = 0._wp
         zrnfm(:,:) = 0._wp
         zh_rnfm(:,:) = 0._wp
         zfr_im(:,:) = 0._wp

      ENDIF

      CALL iom_setkt( kt )                  ! iom_put outside of crs is called at every time step



      ! 5. Clean-up
      ! --------------------------------------------------------
      !
      CALL wrk_dealloc( jpi , jpj , z2d, ze1e2u, ze1e2v )
      CALL wrk_dealloc( jpi , jpj , jpk , z3d, z3d1 )
      CALL wrk_dealloc( jpi , jpj , jpk , zfse3u, zfse3v )
      CALL wrk_dealloc( jpi , jpj , jpk , zfse3t, zfse3w )
      CALL wrk_dealloc( jpi_crs , jpj_crs , jpk , z3dcrs )
      CALL wrk_dealloc( jpi_crs , jpj_crs , z2dcrs )

      DEALLOCATE( z2dcrsu, z2dcrsv, z2dcrsf, zhdivbt )
      DEALLOCATE( zsshub, zsshua, zsshvb, zsshva )
      DEALLOCATE( zee_t, zee_f, zee_u, zee_v )
      DEALLOCATE( zmut, zmuf )
      DEALLOCATE( zcrs_surtaux_wgt, zcrs_surtauy_wgt )
      DEALLOCATE( zhdiv, zvolt_wgt) 
      !
      IF( nn_timing == 1 )   CALL timing_stop('crs_dia_wri')
      !
   END SUBROUTINE crs_dia_wri

   !!======================================================================
END MODULE crsdiawri 
